
import numpy as np
from scipy.linalg import eigh
from numpy import log2
import networkx as nx
from qiskit import QuantumCircuit, Aer, transpile
from qiskit.quantum_info import Statevector, partial_trace, entropy
import pandas as pd

# Define entanglement structure: 6-node MERA-like tree
# Q1=0, Q2=1, Q3=2, I1=3, I2=4, R=5
edges = [(0, 3), (1, 3), (1, 4), (2, 4), (3, 5), (4, 5)]

# Build the quantum circuit
qc = QuantumCircuit(6)
qc.h(range(6))  # Initialize in |+> state
for a, b in edges:
    qc.cz(a, b)  # Apply entangling CZ gates

# Simulate the quantum state
backend = Aer.get_backend("statevector_simulator")
sv = Statevector.from_instruction(qc)

# Define subsystems for entropy calculation
subsystems = {
    "Root (R)": [5],
    "Intermediates (I1, I2)": [3, 4],
    "Upper Layer (I1, I2, R)": [3, 4, 5]
}

# Compute entropy
results = []
for label, kept in subsystems.items():
    traced = [i for i in range(6) if i not in kept]
    rho_sub = partial_trace(sv, traced)
    S = entropy(rho_sub, base=2)
    results.append((label, S))

# Display results
df = pd.DataFrame(results, columns=["Subsystem", "Entropy (S) in bits"])
print(df)
